# -*- coding: utf-8 -*-
import click
import os
import time

_VERSION = 7.0

_F1 = [
    'Node',
    'Coordenada X',
    'Coordenada Y',
    'Coordenada Z',
    'CNAE',
    'Equip de mesura',
    'Codi de tarifa',
    'Zona de qualitat',
    'CUPS',
    'Codi distribuidora',
    'Municipi',
    'Provincia',
    'Connexió',
    'Tensió d\'alimentació',
    'Estat del contracte',
    'Potència contractada',
    'Potència facturada',
    'Potència adscrita',
    'Energia anual activa',
    'Energia anual reactiva',
    'Any d\'informació'
]

_F1_exclude = [
    0, 1, 2, 3, 12, 19
]

_F1BIS = [
    'CUPS',
    'CINI',
    'Instal·lació',
    'Número de lectures',
    'Canvi de titularitat',
    'Baixa del subministre',
    'Any d\'informació'
]

_F2A = [
    'Codi de mercat',
    'Codi companyia distribuidora declarant',
    'Codi companyia distribuidora client',
    'Codi frontera REE',
    'Coordenada X',
    'Coordenada Y',
    'Tensió'
]

_F2B = [
    'Codi de mercat',
    'Codi companyia distribuidora declarant',
    'Municipi',
    'Provincia'
]

_F10 = [
    'Tram',
    'Node inicial',
    'Node final',
    'CINI',
    'Provincia',
    'Nivell de tensió',
    'Longitud',
    'Número de circuits',
    'Tipus',
    'R',
    'X',
    'Intensitat màxima',
    'Estat operació',
    'Codi companyia distribuidora',
    'Propietat',
    'Any d\'informació'
]


@click.command()
@click.option('-d', '--dir', help='Ruta de la carpeta amb els formularis')
def node_check(**kwargs):
    if not kwargs['dir']:
        print "ERROR: falta especificar la ruta de la carpeta amb els" \
              " formularis."
    else:
        ruta = kwargs['dir']
        if os.path.exists(ruta):
            forms = os.listdir(ruta)
            forms.sort()
            buidar_resultats_anteriors(ruta)
            f11 = None
            f10 = None
            f9 = None
            f1 = None
            for form in forms:
                if '_11_' in form and '~' not in form:
                    f11 = ruta+'/'+form
                if '_10_' in form and '~' not in form:
                    f10 = ruta+'/'+form
                if '_9_' in form and '~' not in form:
                    f9 = ruta+'/'+form
                if '_1_' in form and '~' not in form:
                    f1 = ruta+'/'+form
            if f10:
                for form in forms:
                    ruta_completa = ruta+'/'+form
                    if '_1_' in form and '~' not in form:
                        print "Verificant nodes del formulari 1...\n"
                        nodes = check(f10, ruta_completa, 'f1')
                        mostrar_nodes(nodes, 'f1', ruta)
                        print "Verificant CUPS del formulari 1...\n"
                        cups = check_cups(ruta_completa, 'f1')
                        mostrar_cups(cups, 'f1', ruta)
                        print "Verificant camps obligatoris del formulari " \
                              "1...\n"
                        camps = check_obligatoris(ruta_completa, 'f1')
                        mostrar_obligatoris(camps, 'f1', ruta)
                    elif ('_1bis_' in form or '_1BIS_' in form) \
                            and '~' not in form:
                        print "Verificant CUPS del formulari 1bis...\n"
                        cups = check_cups(ruta_completa, 'f1bis')
                        mostrar_cups(cups, 'f1bis', ruta)
                        cups = check_cups_f1bis(ruta_completa, f1)
                        mostrar_cups_f1bis(cups, ruta)
                        print "Verificant camps obligatoris del formulari " \
                              "1bis...\n"
                        camps = check_obligatoris(ruta_completa, 'f1bis')
                        mostrar_obligatoris(camps, 'f1bis', ruta)
                    elif '_2_' in form and '~' not in form:
                        print "Verificant nodes del formulari 2...\n"
                        nodes = check(f10, ruta_completa, 'f2')
                        mostrar_nodes(nodes, 'f2', ruta)
                        print "Verificant CUPS del formulari 2...\n"
                        cups = check_cups(ruta_completa, 'f2')
                        mostrar_cups(cups, 'f2', ruta)
                    elif ('_2a_' in form or '_2A_' in form) and '~' not in form:
                        print "Verificant camps obligatoris del formulari" \
                              "2a..."
                        camps = check_obligatoris(ruta_completa, 'f2a')
                        mostrar_obligatoris(camps, 'f2a', ruta)
                    elif ('_2b_' in form or '_2B_' in form) and '~' not in form:
                        print "Verificant camps obligatoris del formulari" \
                              "2b..."
                        camps = check_obligatoris(ruta_completa, 'f2b')
                        mostrar_obligatoris(camps, 'f2b', ruta)
                    elif '_3_' in form and '~' not in form:
                        print "Verificant nodes del formulari 3...\n"
                        nodes = check(f10, ruta_completa, 'f3')
                        mostrar_nodes(nodes, 'f3', ruta)
                    elif '_6_' in form and '~' not in form:
                        print "Verificant nodes del formulari 6...\n"
                        nodes = check(f10, ruta_completa, 'f6')
                        mostrar_nodes(nodes, 'f6', ruta)
                        print "Verificant CUPS del formulari 6...\n"
                        cups = check_cups(ruta_completa, 'f6')
                        mostrar_cups(cups, 'f6', ruta)
                    elif '_9_' in form and '~' not in form:
                        print "Verificant trams dels formularis 9 i 10...\n"
                        trams, topologia = check_trams_f9_f10(
                            f10, ruta_completa)
                        mostrar_trams(trams, ruta, 'f9_f10', topologia)
                        print "Verificant trams repetits del formulari " \
                              "9...\n"
                        repetits = comprovar_repetits(ruta_completa, 'f9')
                        mostrar_trams(repetits, ruta, 'f9r')
                    elif '_10_' in form and '~' not in form:
                        print "Verificant trams repetits del formulari 10...\n"
                        repetits = comprovar_repetits(ruta_completa, 'f10')
                        mostrar_trams(repetits, ruta, 'f10r')
                        print "Verificant camps obligatoris del formulari " \
                              "10...\n"
                        camps = check_obligatoris(ruta_completa, 'f10')
                        mostrar_obligatoris(camps, 'f10', ruta)
                    elif '_11_' in form and '~' not in form:
                        print "Verificant nodes del formulari 11...\n"
                        nodes = check(f10, ruta_completa, 'f11')
                        mostrar_nodes(nodes, 'f11', ruta)
                    elif '_12_' in form and '~' not in form:
                        # print "Verificant nodes del formulari 12...\n"
                        # nodes = check(f10, ruta_completa, 'f12')
                        # mostrar_nodes(nodes, 'f12', ruta)
                        if f11:
                            print "Verificant CTs del formulari 12bis...\n"
                            cts = comprovar_cts(f11, ruta_completa, 'f12')
                            mostrar_cts(ruta, cts, 'f12')
                    elif ('_12bis_' in form or '_12BIS_' in form) \
                            and '~' not in form:
                        if f11:
                            print "Verificant CTs del formulari 12bis...\n"
                            cts = comprovar_cts(f11, ruta_completa, 'f12bis')
                            mostrar_cts(ruta, cts, 'f12bis')
                        print "Verificant codis de posició del formulari " \
                              "12bis...\n"
                        posicions = comprovar_repetits(ruta_completa, 'f12bis')
                        mostrar_posicions(ruta, posicions)
                    elif ('_13bis_' in form or '_13BIS_' in form) \
                            and '~' not in form:
                        print "Verificant nodes del formulari 13 bis...\n"
                        nodes = check(f10, ruta_completa, 'f13bis')
                        mostrar_nodes(nodes, 'f13bis', ruta)
                    elif '_15_' in form and '~' not in form:
                        print "Verificant nodes del formulari 15...\n"
                        nodes = check(f10, ruta_completa, 'f15')
                        mostrar_nodes(nodes, 'f15', ruta)
                        if f9:
                            print "Verificant trams del formulari 15 amb els" \
                                  " del formulari 9...\n"
                            trams = check_trams_f15(f9, ruta_completa)
                            mostrar_trams(trams, ruta, 'f15')
                    elif '_16_' in form and '~' not in form:
                        print "Verificant nodes del formulari 16...\n"
                        nodes = check(f10, ruta_completa, 'f16')
                        mostrar_nodes(nodes, 'f16', ruta)
                    elif '_17_' in form and '~' not in form:
                        print "Verificant nodes del formulari 17...\n"
                        nodes = check(f10, ruta_completa, 'f17')
                        mostrar_nodes(nodes, 'f17', ruta)
                    elif '_20_' in form and '~' not in form:
                        print "Verificant CUPS del formulari 20...\n"
                        cups = check_cups(ruta_completa, 'f20')
                        mostrar_cups(cups, 'f20', ruta)
                print "\n\nVerificació completada."
            else:
                print "ERROR: no s'ha trobat el formulari f10 a la" \
                      "carpeta especificada."
        else:
            print "ERROR: No s'ha trobat la carpeta de formularis especificada."


def buidar_resultats_anteriors(ruta):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'w') as f:
        f.write('Resultats de la comprovació dels formularis CNMC 4/2015\n\n')
        f.write('Versió: {0}\n\n'.format(_VERSION))
        f.write('Data de comprovació: {0}\n\n'.format(
            time.strftime("%d/%m/%Y")))
        f.write('#######################################################'
                '\n\n\n\n')


def check_obligatoris(fitxer, context):
    camps = {}
    if existeix_fitxer(fitxer):
        with open(fitxer, 'r') as f:
            # j és el numero de linia del fitxer on s'ha trobat que falta un
            # camp
            j = 1
            for linia in f:
                linia = linia.rstrip()
                camps_linia = linia.split(';')
                i = 0
                for camp in camps_linia:
                    if context == 'f1':
                        cups = camps_linia[8]
                        if camp == '' and i not in _F1_exclude:
                            if cups not in camps.keys():
                                camps[cups] = []
                            camps[cups].append(_F1[i])
                    elif context == 'f1bis':
                        cups = camps_linia[0]
                        if camp == '':
                            if cups not in camps.keys():
                                camps[cups] = []
                            camps[cups].append(_F1BIS[i])
                    elif context == 'f2a':
                        if camp == '':
                            if j not in camps.keys():
                                camps[j] = []
                            camps[j].append(_F2A[i])
                    elif context == 'f2b':
                        if camp == '':
                            if j not in camps.keys():
                                camps[j] = []
                            camps[j].append(_F2B[i])
                    elif context == 'f10':
                        tram = camps_linia[0]
                        if camp == '':
                            if tram not in camps.keys():
                                camps[tram] = []
                            if tram == '':
                                if j not in camps.keys():
                                    camps[j] = []
                                camps[j].append(_F10[i])
                            else:
                                camps[tram].append(_F10[i])
                    i += 1
                j += 1
    return camps


def mostrar_obligatoris(camps, form, ruta):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if camps:
            f.write("Camps obligatoris del formulari {0} que falten:"
                    "\n".format(form))
            i = 0
            for elem in camps:
                if len(camps[elem]) > 0:
                    f.write("\n")
                    if form == 'f2a' or form == 'f2b':
                        f.write("\tLínia: {0}\n\n".format(elem))
                    elif form == 'f10':
                        if "Tram" in camps[elem]:
                            f.write("\tLínia: {0}\n\n".format(elem))
                        else:
                            f.write("\tTram: {0}\n\n".format(elem))
                    else:
                        f.write("\tCUPS: {0}\n\n".format(elem))
                    i += 1
                    for c in camps[elem]:
                        f.write("\t\t{0}\n".format(c))
            f.write("\nTotal de línies amb camps sense emplenar a {0}: "
                    "{1}\n\n".format(form, i))
        else:
            f.write("Tots els camps obligatoris del fitxer {0} estàn complets."
                    " OK!\n\n".format(form))


def mostrar_cups_f1bis(cups, ruta):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cups:
            f.write("CUPS del formulari F1 Bis que no apareixen al formulari "
                    "F1:\n\n")
            for elem in cups:
                f.write("\t{0}\n".format(elem))
            f.write("\nTotal CUPS: {0}\n\n".format(len(cups)))
        else:
            f.write("Els cups del formulari F1 Bis coincideixen amb els del"
                    " formulari F1. OK!\n\n")


def check_cups_f1bis(fitxer, f1):
    cups_f1 = []
    cups_f1bis = []
    resultat = []
    if existeix_fitxer(fitxer):
        if existeix_fitxer(f1):
            with open(fitxer) as f1bis:
                for linia in f1bis.readlines():
                    cups_f1bis.append(linia.split(';')[0])
            with open(f1) as f:
                for linia in f.readlines():
                    cups_f1.append(linia.split(';')[8])
            resultat = set(cups_f1bis) - set(cups_f1)
            resultat = list(resultat)
    return resultat


def mostrar_cups(cups, form, ruta):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cups:
            f.write("CUPS duplicats del formulari {0}:\n".format(form))
            i = 0
            for elem in cups:
                if len(cups[elem]) > 1:
                    f.write("\n")
                    f.write("\tCodi: {0}\n\n".format(elem))
                    i += 1
                    for c in cups[elem]:
                        f.write("\t\t{0}\n".format(c))
            f.write("\nTotal CUPS duplicats a {0}: {1}\n\n".format(
                form, i))
        else:
            f.write("No hi ha CUPS duplicats al formulari {0}. "
                    "OK!\n\n".format(form))


def check_cups(fitxer, form):
    cups = {}
    cups_repetits = {}
    if existeix_fitxer(fitxer):
        with open(fitxer, 'r') as f:
            for linia in f.readlines():
                if form == 'f1':
                    cups_complet = linia.split(';')[8]
                elif form == 'f1bis':
                    cups_complet = linia.split(';')[0]
                elif form == 'f2':
                    cups_complet = linia.split(';')[5]
                elif form == 'f6':
                    cups_complet = linia.split(';')[6]
                elif form == 'f20':
                    cups_complet = linia.split(';')[1]
                cups_trim = cups_complet[:18]
                if cups_trim in cups.keys():
                    cups_repetits[cups_trim] = cups[cups_trim]
                    cups_repetits[cups_trim].append(cups_complet)
                else:
                    cups[cups_trim] = [cups_complet]
    return cups_repetits


def mostrar_posicions(ruta, posicions):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if posicions:
            f.write("Posicions duplicades del formulari F12 Bis:\n\n")
            for posicio in posicions:
                f.write("\t{0}\n".format(posicio))
            f.write("\nTotal posicions duplicades F12 Bis: {0}\n\n".format(
                len(posicions)))
        else:
            f.write("Els codis de posició del formulari F12 Bis no estan "
                    "duplicats. OK!\n\n")


def mostrar_cts(ruta, cts, context):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cts:
            f.write("CTs del formulari {0} que no apareixen al formulari "
                    "11:\n\n".format(context))
            for ct in cts:
                f.write("\tEl CT {0} no apareix al formulari F11.\n".format(ct))
            f.write("\nTotal CTs {0}: {1}\n\n".format(context, len(cts)))
        else:
            f.write("Els CTs del formulari {0} apareixen a l'f11. "
                    "OK!\n\n".format(context))


def comprovar_cts(ruta_f11, ruta_fitxer, context):
    cts = []
    index = 0
    cts_f11 = []
    cts_fitxer = []
    if context == 'f12':
        index = 1
    if existeix_fitxer(ruta_f11):
        if existeix_fitxer(ruta_fitxer):
            # Carreguem els CTs de l'F11
            with open(ruta_f11, 'r') as f11:
                for linia in f11.readlines():
                    linia = linia.rstrip()
                    ct = linia.split(';')[1]
                    if ct not in cts_f11:
                        cts_f11.append(ct)
            # Comprovem els CTS amb l'F11
            with open(ruta_fitxer, 'r') as fitxer:
                for linia in fitxer.readlines():
                    linia = linia.rstrip()
                    ct = linia.split(';')[index]
                    cts_fitxer.append(ct)
            cts = set(cts_fitxer) - set(cts_f11)
            cts = list(cts)
    return cts


def comprovar_repetits(ruta_fitxer, context):
    trams = []
    repetits = []
    if existeix_fitxer(ruta_fitxer):
        with open(ruta_fitxer, 'r') as fitxer:
            for linia in fitxer.readlines():
                linia = linia.rstrip()
                if context == 'f9':
                    if ';' not in linia and linia != 'END' and linia != '':
                        if linia not in trams:
                            trams.append(linia)
                        else:
                            repetits.append(linia)
                elif context == 'f10':
                    tram = linia.rsplit(';')[0]
                    if tram not in trams:
                        trams.append(tram)
                    else:
                        repetits.append(tram)
                elif context == 'f12bis':
                    posicio = linia.split(';')[2]
                    if posicio not in trams:
                        trams.append(posicio)
                    else:
                        repetits.append(posicio)
    return repetits


def check_trams_f15(ruta_f9, ruta_f15):
    trams = []
    if existeix_fitxer(ruta_f9):
        if existeix_fitxer(ruta_f15):
            trams_f9 = []
            trams_f15 = []
            # llegim tots els trams de l'F9
            with open(ruta_f9, 'r') as f9:
                for linia in f9.readlines():
                    linia = linia.rstrip()
                    if ';' not in linia and linia != 'END' and linia != '':
                        trams_f9.append(linia)
            # llegim tots els trams de l'F15
            with open(ruta_f15, 'r') as f15:
                for linia in f15.readlines():
                    linia = linia.rstrip()
                    trams_f15.append(linia.split(';')[2])
            # comparem trams de f15 si son a f9
            resta = set(trams_f15) - set(trams_f9)
            trams = [(x, 'f9', 'f15') for x in resta]
    return trams


def check_trams_f9_f10(ruta_f10, ruta_f9):
    trams = []
    if existeix_fitxer(ruta_f10):
        if existeix_fitxer(ruta_f9):
            trams_f9 = []
            trams_f10 = []
            topologia = []
            # llegim tots els trams de l'F9
            with open(ruta_f9, 'r') as f9:
                for linia in f9.readlines():
                    linia = linia.rstrip()
                    if linia == '':
                        topologia.append(trams_f9.pop())
                    elif linia and ';' not in linia and linia != 'END':
                        trams_f9.append(linia)
            # llegim tots els trams de l'F10
            with open(ruta_f10, 'r') as f10:
                for linia in f10.readlines():
                    linia = linia.rstrip()
                    if linia:
                        trams_f10.append(linia.split(';')[0])
            # comparem trams de f9 si son a f10
            resta = set(trams_f9) - set(trams_f10)
            trams = [(x, 'f10', 'f9') for x in resta]
            # comparem trams de f10 si son a f9
            resta = set(trams_f10) - set(trams_f9)
            trams += [(x, 'f9', 'f10') for x in resta]
    return trams, topologia


def mostrar_trams(trams, ruta, context, topologia=None):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        repetit = ''
        if trams:
            if context == 'f9_f10':
                f.write("Trams dels formularis F9 i F10:\n\n")
            elif context == 'f15':
                f.write("Trams del formulari F15:\n\n")
            elif context == 'f9r':
                repetit = 'F9'
                f.write("Trams repetits del formulari F9:\n\n")
            elif context == 'f10r':
                repetit = 'F10'
                f.write("Trams repetits del formulari F10:\n\n")
            for tram in trams:
                if 'r' not in context:
                    f.write("\tEl tram {0} del formulari {1} no apareix al "
                            "formulari {2}\n".format(tram[0], tram[2], tram[1]))
                else:
                    f.write("\tEl tram {0} es repeteix al formulari "
                            "{1}\n".format(tram, repetit))
            f.write('\nTotal trams: {0}\n\n'.format(len(trams)))
        else:
            if context == 'f9_f10':
                f.write("Els trams dels formularis f9 i f10 coincideixen entre "
                        "ells. OK!\n\n")
            elif context == 'f15':
                f.write("Els trams del formulari f15 coincideixen amb els del "
                        "formulari f9. OK!\n\n")
            elif context == 'f10r':
                f.write("No es repeteix cap tram al formulari f10. OK!\n\n")
            elif context == 'f9r':
                f.write("No es repeteix cap tram al formulari f9. OK!\n\n")
        if topologia:
            f.write("Trams del formulari F9 fora de topologia:\n\n")
            for tram in topologia:
                f.write("\tTram fora de topologia: {0}\n".format(tram))
            f.write('\nTotal fora topologia: {0}\n\n'.format(len(topologia)))


def mostrar_nodes(nodes, form, ruta):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if nodes:
            f.write(
                "Nodes del formulari {} que no apareixen al formulari "
                "f10:\n\n".format(form))
            for elem in nodes:
                if form == 'f1' or form == 'f2' or form == 'f6':
                    f.write("\tNode: {0}, CUPS: {1}\n".format(elem[0], elem[1]))
                else:
                    f.write("\t{0}\n".format(elem))
            f.write("\nTotal nodes {0}: {1}\n\n".format(form, len(nodes)))
        else:
            f.write("Els nodes del formulari {0} coincideixen amb els del "
                    "formulari f10. OK!\n\n".format(form))


def check(f10_path, file_path, form):
    nodes = []
    if existeix_fitxer(f10_path):
        if existeix_fitxer(file_path):
            nodes_f10 = []
            with open(f10_path, 'r') as f10:
                node_actual = ''
                for linia_f10 in f10.readlines():
                    linia_f10 = linia_f10.rstrip()
                    if linia_f10 != '':
                        node_actual = linia_f10.split(';')[1]
                        nodes_f10.append(node_actual)
                        node_actual = linia_f10.split(';')[2]
                        nodes_f10.append(node_actual)
            # Ja tenim tots els nodes de F10, ara comprovem que cada node
            # del fitxer a comprovar és dins de F10
            with open(file_path, 'r') as f:
                for linia in f.readlines():
                    if form == 'f1' or form == 'f2' or form == 'f3' \
                            or form == 'f6' or form == 'f11' or form == 'f12' \
                            or form == 'f15' or form == 'f16' or form == 'f17':
                        node_actual = linia.split(';')[0]
                    elif form == 'f13bis':
                        node_actual = linia.split(';')[2]
                    if node_actual not in nodes_f10:
                        cups = 'formulari_sense_cups'
                        if node_actual == '':
                            node_actual = "Node buit"
                        if form == 'f1':
                            cups = linia.split(';')[8]
                        elif form == 'f2':
                            cups = linia.split(';')[3]
                        elif form == 'f6':
                            cups = linia.split(';')[4]
                        if not cups:
                            cups = "Sense CUPS"
                        if cups == 'formulari_sense_cups':
                            nodes.append(node_actual)
                        else:
                            nodes.append((node_actual, cups))
    return nodes


def existeix_fitxer(ruta_fitxer):
    if os.path.exists(ruta_fitxer):
        return True
    else:
        print "El fitxer {0} no existeix.".format(ruta_fitxer)
        return False


if __name__ == '__main__':
    node_check()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Subestacions
"""
from datetime import datetime
import traceback
import sys

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_municipi_from_company

QUIET = False


class SUB(MultiprocessBased):
    def __init__(self, **kwargs):
        super(SUB, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Subestacions'
        self.report_name = 'CNMC INVENTARI SUB'

    def get_sequence(self):
        search_params = []
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCtsSubestacions.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        O = self.connection
        fields_to_read = ['name', 'data_industria', 'data_pm', 'id_municipi',
                          'posicions', 'cini', 'descripcio', 'perc_financament']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                sub = O.GiscedataCtsSubestacions.read(item, fields_to_read)

                if not sub:
                    txt = ("**** ERROR: El ct %s (id:%s) no està en "
                           "giscedata_cts_subestacions.\n" %
                           (sub['name'], sub['id']))

                    if not QUIET:
                        sys.stderr.write(txt)
                        sys.stderr.flush()

                    raise Exception(txt)

                # Calculem any posada en marxa
                data_pm = sub['data_pm']

                if data_pm:
                    data_pm = datetime.strptime(str(data_pm), '%Y-%m-%d')
                    data_pm = data_pm.strftime('%d/%m/%Y')

                comunitat = ''
                id_municipi = None
                if sub['id_municipi']:
                    id_municipi = sub['id_municipi'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)

                if id_municipi:
                    #funció per trobar la ccaa desde el municipi
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat = comunitat_vals['codi']

                output = [
                    '%s' % sub['name'],
                    sub['cini'] or '',
                    sub['descripcio'] or '',
                    '',
                    comunitat,
                    round(100 - int(sub['perc_financament'])),
                    data_pm,
                    '',
                    len(sub['posicions'])
                ]

                self.output_q.put(output)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC AT
"""
from datetime import datetime
import traceback
import math
import time

from libcnmc.core import MultiprocessBased
from libcnmc.utils import format_f, tallar_text


class LAT(MultiprocessBased):
    """
    Class that generates the LAT(1) file of  4131
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(LAT, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies AT'
        self.report_name = 'CNMC INVENTARI AT'
        self.layer = 'LBT\_%'
        self.embarrats = kwargs.pop('embarrats', False)
        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        """
        search_params = [('propietari', '=', True)]
        obj_lat = self.connection.GiscedataAtLinia
        ids = obj_lat.search(search_params, 0, 0, False, {'active_test': False})
        id_lat_emb = []
        if self.embarrats:
            id_lat_emb = obj_lat.search(
                [('name', '=', '1')], 0, 0, False, {'active_test': False})
        return ids + id_lat_emb

    def consumer(self):
        """
        Method that generates the csb file
        :return: List of arrays
        """
        O = self.connection
        fields_to_read = ['baixa', 'data_pm', 'data_industria',
                          'coeficient', 'cini', 'propietari',
                          'tensio_max_disseny', 'name', 'origen',
                          'final', 'perc_financament', 'circuits',
                          'longitud_cad', 'cable', 'cnmc_tipo_instalacion',
                          'data_baixa']
        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)

        static_search_params = [
            ('propietari', '=', True),
            '|', ('data_pm', '=', False), ('data_pm', '<', data_pm_limit),
            '|', ('data_baixa', '=', False), ('data_baixa', '>', data_baixa)
            ]

        #print 'static_search_params:{}'.format(static_search_params)
        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += [
            '|',
            '&', ('active', '=', False), ('data_baixa', '!=', False),
            ('active', '=', True)
        ]
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataAtLinia.read(
                    item, ['trams', 'tensio', 'municipi', 'propietari']
                )
                propietari = linia['propietari'] and '1' or '0'
                search_params = [('linia', '=', linia['id'])]
                search_params += static_search_params
                ids = O.GiscedataAtTram.search(
                    search_params, 0, 0, False, {'active_test': False})
                id_desconegut = O.GiscedataAtCables.search(
                    [('name', '=', 'DESCONEGUT')])

                if not id_desconegut:
                    id_desconegut = O.GiscedataAtCables.search(
                    [('name', '=', 'DESCONOCIDO')])[0]
                for tram in O.GiscedataAtTram.read(ids, fields_to_read):
                    # Comprovar el tipus del cable
                    if 'cable' in tram:
                        cable = O.GiscedataAtCables.read(tram['cable'][0],
                                                     ['tipus'])
                        if not self.embarrats and cable['tipus']:
                            tipus = O.GiscedataAtTipuscable.read(
                                        cable['tipus'][0], ['codi']
                            )
                            # Si el tram tram es embarrat no l'afegim
                            if tipus['codi'] == 'E':
                                continue
                    else:
                        cable = O.GiscedataAtCables.read(
                            id_desconegut, ['tipus'])

                    # Calculem any posada en marxa
                    data_pm = ''
                    if 'data_pm' in tram and tram['data_pm'] and tram['data_pm'] < data_pm_limit:
                        data_pm = datetime.strptime(str(tram['data_pm']),
                                                    '%Y-%m-%d')
                        data_pm = data_pm.strftime('%d/%m/%Y')

                    # Coeficient per ajustar longituds de trams
                    coeficient = tram.get('coeficient',1.0)

                    codi_ccuu = tram.get('cnmc_tipo_instalacion', '')

                    #Agafem la tensió
                    if 'tensio_max_disseny' in tram :
                        tensio = tram['tensio_max_disseny'] / 1000.0
                    elif 'tensio' in linia:
                        tensio = linia['tensio'] / 1000.0
                    else:
                        tensio = 0


                    comunitat = ''
                    if linia['municipi']:
                        ccaa_obj = O.ResComunitat_autonoma
                        id_comunitat = ccaa_obj.get_ccaa_from_municipi(
                            linia['municipi'][0])
                        comunidad = O.ResComunitat_autonoma.read(id_comunitat,
                                                                 ['codi'])
                        if comunidad:
                            comunitat = comunidad[0]['codi']

                    # Agafem el cable de la linia
                    if 'cable' in tram:
                        cable = O.GiscedataAtCables.read(
                            tram['cable'][0], ['intensitat_admisible',
                                               'seccio'])
                    else:
                        cable = O.GiscedataAtCables.read(
                            id_desconegut[0], ['tipus'])

                    #Capacitat
                    if 'intensitat_admisible' in cable:
                        cap = (cable['intensitat_admisible'] * tensio *
                               math.sqrt(3) / 1000.0)
                    else:
                        cap = 0

                    if cap < 1:
                        capacitat = 1
                    else:
                        capacitat = int(round(cap))

                    #Descripció
                    origen = tallar_text(tram['origen'], 50)
                    final = tallar_text(tram['final'], 50)
                    if 'longitud_cad' in tram:
                        longitud = round(tram['longitud_cad'] * coeficient/ 1000.0, 3) or 0.001
                    else:
                        longitud = 0
                    if not origen or not final:
                        res = O.GiscegisEdge.search(
                            [('id_linktemplate', '=', tram['name']),
                             ('layer', 'not ilike', self.layer),
                             ('layer', 'not ilike', 'EMBARRA%BT%')
                             ])
                        if not res or len(res) > 1:
                            edge = {'start_node': (0, '{0}_0'.format(tram.get('name'))),
                                    'end_node': (0, '{0}_1'.format(tram.get('name')))}
                        else:
                            edge = O.GiscegisEdge.read(res[0], ['start_node',
                                                                'end_node'])
                    if tram.get('data_baixa'):
                        if tram.get('data_baixa') > data_pm_limit:
                            fecha_baja = ''
                        else:
                            tmp_date = datetime.strptime(
                                tram.get('data_baixa'), '%Y-%m-%d')
                            fecha_baja = tmp_date.strftime('%d/%m/%Y')
                    else:
                       fecha_baja = ''

                    if data_pm:
                        dpm = time.strptime(data_pm, '%d/%m/%Y')
                        db = time.strptime(data_baixa, '%Y-%m-%d')
                        if dpm > db:
                            estado = 2
                        else:

                            estado = 0
                    else:
                        estado = 0

                    output = [
                        'A{0}'.format(tram['name']),
                        tram.get('cini', '') or '',
                        origen or edge['start_node'][1],
                        final or edge['end_node'][1],
                        codi_ccuu or '',
                        comunitat,
                        comunitat,
                        format_f(round(100 - int(tram.get('perc_financament', 0) or 0))),
                        data_pm,
                        fecha_baja or '',
                        tram.get('circuits', 1) or 1,
                        1,
                        format_f(tensio),
                        format_f(longitud, 3),
                        format_f(cable.get('intensitat_admisible', 0) or 0),
                        format_f(cable.get('seccio', 0) or 0),
                        capacitat,
                        propietari,
                        estado
                    ]

                    self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f, tallar_text
from libcnmc.core import MultiprocessBased


class F10AT(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F10AT, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F10AT - CTS'
        self.base_object = 'AT'
        self.layer = 'LBT\_%'
        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']

    def get_sequence(self):
        search_params = [('name', '!=', '1')]
        lines_id = self.connection.GiscedataAtLinia.search(search_params)
        search_params = [('name', '=', '1')]
        fict_line_id = self.connection.GiscedataAtLinia.search(
            search_params, 0, 0, False, {'active_test': False})
        return lines_id + fict_line_id


    def get_provincia(self, id_prov):
        o = self.connection
        res = ''
        ine = o.ResCountryState.read(id_prov, ['code'])['code']
        if ine:
            res = ine
        return res

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'name', 'cini', 'circuits', 'longitud_cad', 'linia', 'origen',
            'final', 'coeficient', 'cable', 'tensio_max_disseny'
        ]
        data_pm_limit = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        static_search_params = [
            ('cini', '!=', '0000000'),
            '|',
            ('data_pm', '=', False),
            ('data_pm', '<', data_pm_limit),
            '|',
            ('data_baixa', '>', data_baixa),
            ('data_baixa', '=', False),
        ]
        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += ['|',
                                 '&', ('active', '=', False),
                                      ('data_baixa', '!=', False),
                                 ('active', '=', True)]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                self.progress_q.put(item)
                linia = o.GiscedataAtLinia.read(
                    item, ['trams', 'tensio', 'municipi', 'propietari',
                           'provincia']
                )

                o_prop = linia['propietari'] and '1' or '0'
                search_params = [('id', 'in', linia['trams'])]
                search_params += static_search_params
                ids = o.GiscedataAtTram.search(
                    search_params, 0, 0, False, {'active_test': False})
                for at in o.GiscedataAtTram.read(ids, fields_to_read):
                    # Coeficient per ajustar longituds de trams
                    coeficient = at['coeficient'] or 1.0
                    # Comprovar el tipus del cable
                    fields_to_read_cable = [
                        'tipus', 'resistencia', 'reactancia',
                        'intensitat_admisible'
                    ]
                    cable = o.GiscedataAtCables.read(at['cable'][0],
                                                     fields_to_read_cable)
                    o_tipus = o.GiscedataAtTipuscable.read(cable['tipus'][0],
                                                           ['codi'])['codi']
                    # Si el tram tram es embarrat amb una longitud > 100
                    # no l'afegim
                    if o_tipus == 'E' and at['longitud_cad'] > 100:
                        continue
                    if o_tipus == 'E':
                        o_tipus = 'S'
                    #Agafem la tensió
                    o_nivell_tensio = (
                        (at['tensio_max_disseny'] or linia['tensio']))
                    o_nivell_tensio = format_f(
                        float(o_nivell_tensio) / 1000.0, 3)
                    o_tram = 'A%s' % at['name']
                    res = o.GiscegisEdge.search(
                        [('id_linktemplate', '=', at['name']),
                         ('layer', 'not ilike', self.layer),
                         ('layer', 'not ilike', 'EMBARRA%BT%')
                         ])
                    if not res or len(res) > 1:
                        edge = {'start_node': (0, '%s_0' % at['name']),
                                'end_node': (0, '%s_1' % at['name'])}
                    else:
                        edge = o.GiscegisEdge.read(res[0], ['start_node',
                                                            'end_node'])
                    o_node_inicial = tallar_text(edge['start_node'][1], 20)
                    o_node_final = tallar_text(edge['end_node'][1], 20)
                    o_cini = at['cini']
                    o_provincia = ''
                    if linia['provincia']:
                        o_provincia = self.get_provincia(linia['provincia'][0])
                    o_longitud = format_f(
                            round(float(at['longitud_cad']) *
                                  coeficient / 1000.0, 3) or 0.001, decimals=3)
                    o_num_circuits = at['circuits']
                    o_r = format_f(
                        cable['resistencia'] * (float(at['longitud_cad']) *
                                                coeficient / 1000.0) or 0.0,
                        decimals=6)
                    o_x = format_f(
                        cable['reactancia'] * (float(at['longitud_cad']) *
                                               coeficient / 1000.0) or 0.0,
                        decimals=6)
                    o_int_max = format_f(
                        cable['intensitat_admisible'] or 0.0, decimals=3)
                    o_op_habitual = 1  # Tots son actius
                    o_cod_dis = 'R1-%s' % self.codi_r1[-3:]
                    o_any = self.year

                    self.output_q.put([
                        o_tram,
                        o_node_inicial,
                        o_node_final,
                        o_cini,
                        o_provincia,
                        o_nivell_tensio,
                        o_longitud,
                        o_num_circuits,
                        o_tipus,
                        o_r,
                        o_x,
                        o_int_max,
                        o_op_habitual,
                        o_cod_dis,
                        o_prop,
                        o_any
                    ])
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC AT
"""
from datetime import datetime
import traceback
import math

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_expedient, tallar_text


class LAT(MultiprocessBased):
    def __init__(self, **kwargs):
        super(LAT, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies AT'
        self.report_name = 'CNMC INVENTARI AT'

    def get_sequence(self):
        search_params = [('name', '!=', '1'), ('propietari', '=', True)]
        return self.connection.GiscedataAtLinia.search(search_params)

    def consumer(self):
        O = self.connection
        fields_to_read = ['baixa', 'data_pm', 'data_industria',
                          'coeficient', 'cini', 'propietari',
                          'tensio_max_disseny', 'name', 'origen',
                          'final', 'perc_financament', 'circuits',
                          'longitud_cad', 'cable', 'expedients_ids']
        data_pm_limit = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        static_search_params = [('propietari', '=', True),
                                '|', ('data_pm', '=', False),
                                     ('data_pm', '<', data_pm_limit),
                                '|', ('data_baixa', '>', data_baixa),
                                     ('data_baixa', '=', False),
                                ]
        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += ['|',
                                 '&', ('active', '=', False),
                                      ('data_baixa', '!=', False),
                                 ('active', '=', True)]
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataAtLinia.read(
                    item, ['trams', 'tensio', 'municipi']
                )
                search_params = [('id', 'in', linia['trams'])]
                search_params += static_search_params
                ids = O.GiscedataAtTram.search(
                    search_params, 0, 0, False, {'active_test': False})
                for tram in O.GiscedataAtTram.read(ids, fields_to_read):
                    #Comprovar el tipus del cable
                    cable = O.GiscedataAtCables.read(tram['cable'][0],
                                                     ['tipus'])
                    tipus = O.GiscedataAtTipuscable.read(cable['tipus'][0],
                                                         ['codi'])
                    #Si el tram tram es embarrat no l'afegim
                    if tipus['codi'] == 'E':
                        continue

                    # Calculem any posada en marxa
                    data_pm = ''
                    if tram['data_pm'] and tram['data_pm'] < data_pm_limit:
                        data_pm = datetime.strptime(str(tram['data_pm']),
                                                    '%Y-%m-%d')
                        data_pm = data_pm.strftime('%d/%m/%Y')

                    # Coeficient per ajustar longituds de trams
                    coeficient = tram['coeficient'] or 1.0

                    tipus_inst_id = O.Giscedata_cnmcTipo_instalacion.search(
                        [('cini', '=', tram['cini'])])
                    codigo = O.Giscedata_cnmcTipo_instalacion.read(
                        tipus_inst_id, ['codi'])
                    if codigo:
                        codi = codigo[0]
                    else:
                        codi = {'codi': ' '}

                    #Agafem la tensió
                    tensio = tram['tensio_max_disseny'] or linia['tensio']

                    comunitat = ''
                    if linia['municipi']:
                        id_comunitat = O.ResComunitat_autonoma.get_ccaa_from_municipi(
                            linia['municipi'][0])
                        comunidad = O.ResComunitat_autonoma.read(id_comunitat,
                                                                 ['codi'])
                        if comunidad:
                            comunitat = comunidad[0]['codi']

                    # Agafem el cable de la linia
                    cable = O.GiscedataAtCables.read(tram['cable'][0], [
                        'intensitat_admisible', 'seccio'])

                    #Capacitat
                    cap = round(
                        (cable['intensitat_admisible'] * tensio *
                         math.sqrt(3))/1000000, 3)
                    if cap < 1:
                        capacitat = 1
                    else:
                        capacitat = int(round(cap))

                    #Descripció
                    origen = tallar_text(tram['origen'], 50)
                    final = tallar_text(tram['final'], 50)

                    output = [
                        'A%s' % tram['name'],
                        tram['cini'] or '',
                        origen or '',
                        final or '',
                        codi['codi'] or '',
                        comunitat,
                        comunitat,
                        round(100 - int(tram['perc_financament'])),
                        data_pm,
                        '',
                        tram['circuits'] or 1,
                        1,
                        round(tram['longitud_cad'] * coeficient / 1000.0,
                              3) or 0,
                        cable['seccio'],
                        capacitat
                    ]

                    self.output_q.put(output)

            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f
from libcnmc.core import MultiprocessBased


class F13bis(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F13bis, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F13 bis - CTS'
        self.base_object = 'CTS'

    def get_sequence(self):
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params = [('ct_id.propietari', '=', True),
                         '|', ('ct_id.data_pm', '=', False),
                         ('ct_id.data_pm', '<', data_pm),
                         '|', ('ct_id.data_baixa', '>', data_baixa),
                         ('ct_id.data_baixa', '=', False),
                         ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('ct_id.active', '=', False),
                          ('ct_id.data_baixa', '!=', False),
                          ('ct_id.active', '=', True)]
        # Revisem que tingui un parc assignat
        search_params += [('parc_id', '!=', False)]
        return self.connection.GiscedataCtsSubestacions.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_subestacio(self, sub_id):
        o = self.connection
        sub = o.GiscedataCtsSubestacions.read(sub_id, ['ct_id', 'cini'])
        ct_id = sub['ct_id'][0]
        cini = sub['cini']
        bloc_ids = o.GiscegisBlocsCtat.search([('ct', '=', ct_id)])
        node = ''
        if bloc_ids:
            bloc = o.GiscegisBlocsCtat.read(bloc_ids[0], ['node'])
            node = bloc['node'][1]
        else:
            print "ct id: {}".format(ct_id)
        return {'node': node, 'cini': cini}

    def get_parc(self, parc_id, data):
        o = self.connection
        res = ''
        if data == 'codi':
            res = o.GiscedataParcs.read(parc_id, ['name'])['name']
        elif data == 'tipus':
            res = o.GiscedataParcs.read(parc_id, ['tipus'])['tipus'] - 1
        elif data == 'tensio':
            tensio_id = o.GiscedataParcs.read(
                parc_id, ['tensio_id'])['tensio_id'][0]
            res = o.GiscedataTensionsTensio.read(
                tensio_id, ['tensio'])['tensio']
        return res

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'id', 'propietari', 'name', 'parc_id'
        ]
        dict_cts = {}
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                self.progress_q.put(item)
                sub = o.GiscedataCtsSubestacions.read(
                    item, fields_to_read
                )
                o_subestacio = sub['name']
                # o_parc = sub['subestacio_id'][1] + "-" + sub['tensio'][1]
                o_parc = self.get_parc(sub['parc_id'][0], 'codi')
                subestacio = self.get_subestacio(sub['id'])
                o_node = subestacio['node']
                o_cini = subestacio['cini']
                o_tipus = self.get_parc(sub['parc_id'][0], 'tipus')
                tensio = self.get_parc(sub['parc_id'][0], 'tensio')
                o_tensio = format_f(
                    float(tensio) / 1000.0, decimals=3)
                o_prop = int(sub['propietari'])
                o_any = self.year
                insert = True
                if o_subestacio not in dict_cts.keys():
                    dict_cts[o_subestacio] = [o_tensio]
                else:
                    llista_valors = dict_cts[o_subestacio]
                    for valor in llista_valors:
                        if valor == o_tensio:
                            insert = False
                    if insert:
                        dict_cts[o_subestacio].append(o_tensio)

                if insert:
                    self.output_q.put([
                        o_subestacio,
                        o_parc,
                        o_node,
                        o_cini,
                        o_tipus,
                        o_tensio,
                        o_prop,
                        o_any
                    ])
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC BT
"""
from datetime import datetime
import traceback
import math
import sys

from libcnmc.core import MultiprocessBased
from libcnmc.utils import format_f, tallar_text

QUIET = False


class LBT(MultiprocessBased):
    def __init__(self, **kwargs):
        super(LBT, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies BT'
        self.report_name = 'CNMC INVENTARI BT'
        self.embarrats = kwargs.pop('embarrats', False)

    def get_sequence(self):

        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params = []
        if not self.embarrats:
            search_params += [('cable.tipus.codi', '!=', 'E')]
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        O = self.connection
        count = 0
        fields_to_read = ['name', 'municipi', 'data_pm', 'ct',
                          'coeficient', 'cini', 'perc_financament',
                          'longitud_cad', 'cable', 'voltatge', 'data_alta',
                          'propietari', 'cnmc_tipo_instalacion']
        while True:
            try:
                count += 1
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataBtElement.read(item, fields_to_read)

                res = O.GiscegisEdge.search([('id_linktemplate', '=',
                                              linia['name']),
                                             ('layer', 'ilike', '%BT%')])
                if not res:
                    if not QUIET:
                        sys.stderr.write(
                            "**** ERROR: l'element %s (id:%s) no està en "
                            "giscegis_edges.\n" % (linia['name'], linia['id']))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '%s_0' % linia['name']),
                            'end_node': (0, '%s_1' % linia['name'])}
                elif len(res) > 1:
                    if not QUIET:
                        sys.stderr.write("**** ERROR: l'element %s (id:%s) "
                                         "està més d'una vegada a "
                                         "giscegis_edges. %s\n" %
                                         (linia['name'], linia['id'], res))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '%s_0' % linia['name']),
                            'end_node': (0, '%s_1' % linia['name'])}
                else:
                    edge = O.GiscegisEdge.read(res[0], ['start_node',
                                                        'end_node'])
                comunitat = ''
                if linia['municipi']:
                    ccaa_obj = O.ResComunitat_autonoma
                    id_comunitat = ccaa_obj.get_ccaa_from_municipi(
                        linia['municipi'][0])
                    id_comunitat = id_comunitat[0]
                    comunidad = ccaa_obj.read(id_comunitat, ['codi'])
                    if comunidad:
                        comunitat = comunidad['codi']
                data_pm = ''
                if linia['data_pm']:
                    data_pm_linia = datetime.strptime(str(linia['data_pm']),
                                                      '%Y-%m-%d')
                    data_pm = data_pm_linia.strftime('%d/%m/%Y')

                # Coeficient per ajustar longituds de trams
                coeficient = linia['coeficient'] or 1.0

                try:
                    tensio = (int(linia['voltatge']) / 1000.0)
                except:
                    tensio = 0.0

                propietari = linia['propietari'] and '1' or '0'

                codi = linia['cnmc_tipo_instalacion']
                # Agafem el cable de la linia
                if linia['cable']:
                    cable = O.GiscedataBtCables.read(linia['cable'][0], [
                        'intensitat_admisible', 'seccio'])
                else:
                    cable = {'seccio': 0, 'intensitat_admisible': 0}

                intensitat = cable['intensitat_admisible']
                #Capacitat
                capacitat = round(
                    (cable['intensitat_admisible'] * int(linia['voltatge'])
                     * math.sqrt(3)) / 1000, 3)

                if not capacitat:
                    capacitat = 1.0

                #Descripció
                origen = tallar_text(edge['start_node'][1], 50)
                final = tallar_text(edge['end_node'][1], 50)

                longitud = round(linia['longitud_cad'] * coeficient / 1000.0,
                                 3) or 0.001

                output = [
                    'B%s' % linia['name'],
                    linia['cini'] or '',
                    origen or '',
                    final or '',
                    codi or '',
                    comunitat,
                    comunitat,
                    format_f(round(100 - int(linia['perc_financament']))),
                    data_pm or '',
                    1,
                    1,
                    format_f(tensio),
                    format_f(longitud, 3),
                    format_f(intensitat),
                    format_f(cable['seccio']),
                    format_f(capacitat),
                    propietari
                ]

                self.output_q.put(output)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f, get_norm_tension
from libcnmc.core import MultiprocessBased


class F14(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F14, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F14 - CTS'
        self.base_object = 'CTS'

    def get_sequence(self):
        search_params = [
            ('reductor', '=', True),
            ('id_estat.cnmc_inventari', '=', True)
        ]
        return self.connection.GiscedataTransformadorTrafo.search(
            search_params
        )

    def get_estat(self, estat_id):
        o = self.connection
        estat = o.GiscedataTransformadorEstat.read(estat_id, ['codi'])
        if estat['codi'] != 1:
            return 0
        else:
            return 1

    def get_costat_alta(self, trafo):
        o = self.connection
        res = ''
        if trafo['conexions']:
            con = o.GiscedataTransformadorConnexio.read(trafo['conexions'][0])
            tensio = con['tensio_primari']
            res = get_norm_tension(o, tensio)
        return res

    def get_costat_baixa(self, trafo):
        o = self.connection
        res = ''
        if trafo['conexions']:
            con = o.GiscedataTransformadorConnexio.read(trafo['conexions'][0])
            tensio = con['tensio_b1']
            res = get_norm_tension(o, tensio)
        return res


    def consumer(self):
        o = self.connection
        fields_to_read = [
            'ct', 'name', 'cini', 'potencia_nominal', 'propietari', 'id_estat',
            'conexions', 'energia_anual', 'potencia_activa',
            'potencia_reactiva', 'perdues_buit', 'perdues_curtcircuit_nominal'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                self.progress_q.put(item)
                trafo = o.GiscedataTransformadorTrafo.read(
                    item, fields_to_read
                )
                o_subestacio = trafo['ct'][1]
                o_maquina = trafo['name']
                o_cini = trafo['cini']
                o_costat_alta = self.get_costat_alta(trafo)
                o_costat_baixa = self.get_costat_baixa(trafo)
                o_pot_maquina = format_f(
                    float(trafo['potencia_nominal']) / 1000.0, decimals=3)
                o_pot_activa = format_f(
                    float(trafo['potencia_activa']), decimals=3)
                o_pot_reactiva = format_f(
                    float(trafo['potencia_reactiva']), decimals=3)
                o_energia_anual = format_f(
                    float(trafo['energia_anual']), decimals=3)
                o_perdues = format_f(
                    float(trafo['perdues_buit']), decimals=3)
                o_perdues_nominal = format_f(
                    float(
                        trafo['perdues_curtcircuit_nominal']
                    ), decimals=3)
                o_propietat = int(trafo['propietari'])
                o_estat = self.get_estat(trafo['id_estat'][0])
                o_any = self.year

                self.output_q.put([
                    o_subestacio,
                    o_maquina,
                    o_cini,
                    o_costat_alta,
                    o_costat_baixa,
                    o_pot_maquina,
                    o_pot_activa,
                    o_pot_reactiva,
                    o_energia_anual,
                    o_perdues,
                    o_perdues_nominal,
                    o_propietat,
                    o_estat,
                    o_any
                ])
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



# -*- coding: utf-8 -*-
from datetime import datetime
from multiprocessing import Manager
import traceback

from libcnmc.core import MultiprocessBased

resultats = {}


class F3(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F3, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        manager = Manager()
        self.cts = manager.dict()
        self.base_object = 'Clients'
        self.report_name = 'F3 - Clients'

    def get_sequence(self):
        data = '%s-12-31' % self.year
        search_params = ['&',
                         ('data_inici', '<=', data),
                         ('data_final', '>=', data)]
        ids = self.connection.GiscedataPolissaModcontractual.search(
            search_params, 0, 0, False, {'active_test': False})
        for item in ids:
            self.escriure(item, 'comer')
            distri = self.get_distri()
        self.escriure(distri, 'distri')
        res = []
        for elem in resultats:
            tupla = (elem, resultats[elem])
            res.append(tupla)
        return res

    def get_distri(self):
        o = self.connection
        res = None
        id_distri = o.ResCompany.search([])
        distri = o.ResCompany.read(id_distri, ['partner_id'])
        if distri:
            res = distri[0]['partner_id'][0]
        return res

    def escriure(self, item, mode):
        fields_to_read_modcon = [
            'polissa_id'
        ]
        fields_to_read_polissa = [
            'comercialitzadora', 'name'
        ]
        fields_to_read_comer = [
            'vat', 'name', 'ref2',
        ]
        o = self.connection
        modcon = o.GiscedataPolissaModcontractual.read(
            item, fields_to_read_modcon)
        comer_id = None
        if modcon and mode == 'comer':
            polissa = o.GiscedataPolissa.read(
                modcon['polissa_id'], fields_to_read_polissa)
            if polissa:
                comercialitzadora = polissa[0]['comercialitzadora'][0]
                comer_id = o.ResPartner.search(
                    [('id', '=', comercialitzadora)])[0]
        elif mode == 'distri':
            comer_id = item
        if comer_id:
            comer = o.ResPartner.read(
                comer_id, fields_to_read_comer)
            if comer and mode != 'distri':
                if comer['name'] not in resultats.keys():
                    resultats[comer['name']] = {}
                    resultats[comer['name']]['total_clients'] = 0
                    resultats[comer['name']]['vat'] = comer['vat']
                    resultats[comer['name']]['ref2'] = comer['ref2']
                resultats[comer['name']]['total_clients'] += 1
            elif comer and mode == 'distri':
                self.output_q.put([
                    comer['vat'],
                    comer['name'],
                    comer['ref2'],
                    self.get_total_clients(resultats),
                    0,
                    0,
                    self.year
                ])

    def get_total_clients(self, resultats):
        res = 0
        for elem in resultats:
            res += resultats[elem]['total_clients']
        return res

    def consumer(self):
        # generar linies

        while True:
            item = self.input_q.get()
            self.progress_q.put(item)
            try:
                self.output_q.put([
                    item[1]['vat'],
                    item[0],
                    item[1]['ref2'],
                    item[1]['total_clients'],
                    0,
                    0,
                    self.year
                ])
            except Exception, e:
                print e
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Equips de fiabilitat
"""
from datetime import datetime
import traceback

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_expedient


class FIA(MultiprocessBased):
    def __init__(self, **kwargs):
        super(FIA, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies FIA'
        self.report_name = 'CNMC INVENTARI FIA'

    def get_sequence(self):
        search_params = [('inventari', '=', 'fiabilitat')]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCellesCella.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        O = self.connection
        fields_to_read = ['name', 'cini', 'data_baixa', 'tipus_element',
                          'installacio', 'expedients', 'data_pm']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                cll = O.GiscedataCellesCella.read(item, fields_to_read)

                #Comprovar si es tipus fiabilitat
                if cll['tipus_element']:

                    cllt = O.GiscedataCellesTipusElement.read(
                        cll['tipus_element'][0], ['name'])

                if cll['cini']:
                    #Busquem per la penúltima lletra
                    pos_cini = cll['cini'][5]
                    if pos_cini == '1':
                        codi = 174
                    elif pos_cini == '2':
                        codi = 177
                    elif pos_cini == '3':
                        codi = 179
                    elif pos_cini == '4':
                        codi = 181
                    elif pos_cini == '5':
                        codi = 182
                    elif pos_cini == '6':
                        codi = 183
                    elif pos_cini == '7':
                        codi = 187
                    else:
                        codi = 0
                else:
                    codi = 0

                #Instal·lació a la que pertany
                cllinst = cll['installacio'].split(',')

                data_pm = ''
                if cll['data_pm']:
                    data_pm_ct = datetime.strptime(str(cll['data_pm']),
                                                   '%Y-%m-%d')
                    data_pm = data_pm_ct.strftime('%d/%m/%Y')

                #Per trobar la comunitat autonoma
                ccaa = ''
                #Comprovo si la cella pertany a ct o lat per trobar la ccaa
                if cllinst[0] == 'giscedata.cts':
                    ct_vals = O.GiscedataCts.read(int(cllinst[1]),
                                                  ['id_municipi'])
                    if ct_vals['id_municipi']:
                        id_municipi = ct_vals['id_municipi'][0]

                elif cllinst[0] == 'giscedata.at.suport':
                    id_linia = O.GiscedataAtSuport.read(int(cllinst[1]),
                                                        ['linia'])
                    lat_vals = O.GiscedataAtLinia.read(
                        int(id_linia['linia'][0]), ['municipi'])
                    if lat_vals['municipi']:
                        id_municipi = lat_vals['municipi'][0]

                if id_municipi:
                    ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi(
                        id_municipi)[0]

                output = [
                    '%s' % cll['name'],
                    cll['cini'] or '',
                    cllt['name'] or '',
                    codi or '',
                    ccaa or '',
                    data_pm,
                    cll['data_baixa'] or ''
                ]
                self.output_q.put(output)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Generació dels CINIS de maquines
"""
from datetime import datetime
import traceback

from libcnmc.core import MultiprocessBased


class CINIPOS(MultiprocessBased):
    def __init__(self, **kwargs):
        super(CINIPOS, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'CINIPOS'
        self.report_name = 'CNMC INVENTARI CINIPOS'

    def get_sequence(self):
        search_params = []
        return self.connection.GiscedataCtsSubestacionsPosicio.search(
            search_params)

    def consumer(self):
        O = self.connection
        fields_to_read = ['cini', 'tensio', 'interruptor', 'tipus_posicio',
                          'barres']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                cini = ''
                posicio = O.GiscedataCtsSubestacionsPosicio.read(
                    item, fields_to_read)

                if not posicio['cini']:
                    #Els cinis de mÃ quines tots comenÃ§en amb I27
                    cini_ = "I28"
                    #Per completar el tercer digit
                    codi3 = ''
                    try:
                        id_tensio = posicio['tensio'][0]
                        obj_tensio = O.GiscedataTensionsTensio.read(id_tensio,
                                                                    ['tensio'])
                        tensio = int(obj_tensio['tensio'])
                    except:
                        print "Error a {}, la tensio de la posicio no " \
                              "es un enter", format(item)
                        continue

                    if 110000 <= tensio < 220000:
                        codi3 = '2'
                    elif 36000 <= tensio < 110000:
                        codi3 = '3'
                    elif 1000 <= tensio < 36000:
                        codi3 = '4'
                    #Per completar el quart digit
                    codi4 = ''
                    if posicio['interruptor']:
                        #els codis  de cini coincideixen amb els que tenim
                        # d'interruptor
                        codi4 = '%s' % posicio['interruptor']
                    #Per completar el cinque digit
                    codi5 = ''
                    if posicio['tipus_posicio'] == 'B':
                        codi5 = '2'
                    elif posicio['tipus_posicio'] == 'C':
                        codi5 = '1'
                    elif posicio['tipus_posicio'] == 'H':
                        codi5 = '3'
                    #Per completar el sise digit
                    codi6 = ''
                    if posicio['barres']:
                        codi6 = '%s' % posicio['barres']

                    cini = cini_ + codi3 + codi4 + codi5 + codi6
                    #El write del cini en cada un dels trafos
                    O.GiscedataTransformadorTrafo.write([item], {'cini': cini})

                self.output_q.put(cini)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


from cnmcmodel import CNMCModel


class F1Res4771(CNMCModel):
    """
        Class for second file of resolution 4771(LAT)
    """

    fields = [
        'identificador',
        'cini',
        'origen',
        'destino',
        'codigo_tipo_linea',
        'codigo_ccaa_1',
        'codigo_ccaa_2',
        'participacion',
        'fecha_aps',
        'numero_circuitos',
        'numero_conductores',
        'nivel_tension',
        'longitud',
        'intensidad_maxima',
        'seccion',
        'capacidad',
        'propiedad'
        ]

    @property
    def ref(self):
        return self.store['identificador'][1:]

    def __cmp__(self, other):
        comp_fields = [
            'longitud', 'cini', 'seccion', 'capacidad', 'fecha_aps'
        ]
        if self.diff(other, comp_fields):
            return True
        else:
            return False








from cnmcmodel import CNMCModel


class F6Res4771(CNMCModel):
    """
        Class for sixth file of resolution 4771(Despatxos)
    """

    fields = [
        'identificador',
        'cini',
        'denominacion',
        'anio_aps',
        'valor_inversion'
    ]

    @property
    def ref(self):
        return self.store['identificador'][1:]

    def __cmp__(self, other):
        # TODO: Add comparsion fields
        comp_fields = []
        if self.diff(other, comp_fields):
            return True
        else:
            return False








from collections import namedtuple
import json


class CNMCModel(object):
    fields = [
        'id',
        'cini',
        'tipus_instalacio',
    ]

    @property
    def ref(self):
        return 1

    def __init__(self, *values, **kwvalues):
        stored = namedtuple('{0}_store'.format(self.__classname__), self.fields)
        self.store = stored(*values, **kwvalues)

    def dump(self, out_format='json'):
        if out_format == 'json':
            return json.dumps(self.store._asdict())
        else:
            return list(self.store)

    def diff(self, obj, fields=None):
        assert isinstance(obj, self.__class__)
        diffs = {}
        if fields is None:
            fields = self.store._fields
        for field in fields:
            self_value = getattr(self, field)
            other_value = getattr(obj, field)
            if self_value != other_value:
                diffs[field] = (self_value, other_value)
        return diffs

    def __cmp__(self, other):
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC BT
"""
from datetime import datetime
from dateutil import parser
import traceback
import math
import sys

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_expedient, tallar_text

QUIET = False


class LBT(MultiprocessBased):
    def __init__(self, **kwargs):
        super(LBT, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies BT'
        self.report_name = 'CNMC INVENTARI BT'

    def get_sequence(self):

        search_params = [('cable.tipus.codi', '!=', 'E')]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        O = self.connection
        count = 0
        fields_to_read = ['name', 'municipi', 'data_pm', 'ct',
                          'coeficient', 'cini', 'perc_financament',
                          'longitud_cad', 'cable', 'voltatge', 'data_alta']
        while True:
            try:
                count += 1
                item = self.input_q.get()
                self.progress_q.put(item)
                
                linia = O.GiscedataBtElement.read(item, fields_to_read)

                res = O.GiscegisEdge.search([('id_linktemplate', '=',
                                              linia['name']),
                                             ('layer', 'ilike', '%BT%')])
                if not res:
                    if not QUIET:
                        sys.stderr.write(
                            "**** ERROR: l'element %s (id:%s) no està en "
                            "giscegis_edges.\n" % (linia['name'], linia['id']))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '%s_0' % linia['name']),
                            'end_node': (0, '%s_1' % linia['name'])}
                elif len(res) > 1:
                    if not QUIET:
                        sys.stderr.write("**** ERROR: l'element %s (id:%s) "
                                         "està més d'una vegada a "
                                         "giscegis_edges. %s\n" %
                                         (linia['name'], linia['id'], res))
                        sys.stderr.flush()
                    edge = {'start_node': (0, '%s_0' % linia['name']),
                            'end_node': (0, '%s_1' % linia['name'])}
                else:
                    edge = O.GiscegisEdge.read(res[0], ['start_node',
                                                        'end_node'])
                comunitat = ''
                if linia['municipi']:
                    id_comunitat = O.ResComunitat_autonoma.get_ccaa_from_municipi(
                        linia['municipi'][0])
                    id_comunitat = id_comunitat[0]
                    comunidad = O.ResComunitat_autonoma.read(id_comunitat,
                                                             ['codi'])
                    if comunidad:
                        comunitat = comunidad['codi']
                data_pm = ''
                if linia['data_pm']:
                    data_pm_linia = datetime.strptime(str(linia['data_pm']),
                                                   '%Y-%m-%d')
                    data_pm = data_pm_linia.strftime('%d/%m/%Y')

                # Coeficient per ajustar longituds de trams
                coeficient = linia['coeficient'] or 1.0

                # Afagem el tipus de instalacio
                tipus_inst_id = O.Giscedata_cnmcTipo_instalacion.search(
                    [('cini', '=', linia['cini'])])
                codigo = O.Giscedata_cnmcTipo_instalacion.read(
                    tipus_inst_id, ['codi'])
                if codigo:
                    codi = codigo[0]
                else:
                    codi = {'codi': ''}

                # Agafem el cable de la linia
                if linia['cable']:
                    cable = O.GiscedataBtCables.read(linia['cable'][0], [
                        'intensitat_admisible', 'seccio'])
                else:
                    cable = {'seccio': 0, 'intensitat_admisible': 0}

                #Capacitat
                cap = round(
                    (cable['intensitat_admisible'] * int(linia['voltatge'])
                     * math.sqrt(3))/1000000, 3)
                if cap < 1:
                    capacitat = 1
                else:
                    capacitat = int(round(cap))

                #Descripció
                origen = tallar_text(edge['start_node'][1], 50)
                final = tallar_text(edge['end_node'][1], 50)

                output = [
                    'B%s' % linia['name'],
                    linia['cini'] or '',
                    origen or '',
                    final or '',
                    codi['codi'] or '',
                    comunitat,
                    comunitat,
                    round(100 - int(linia['perc_financament'])),
                    data_pm or '',
                    '',
                    1,
                    1,
                    round(linia['longitud_cad'] * coeficient / 1000.0, 3) or 0,
                    cable['seccio'],
                    capacitat

                ]

                self.output_q.put(output)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Equips de fiabilitat
"""
from datetime import datetime
import traceback

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_expedient


class FIA(MultiprocessBased):
    def __init__(self, **kwargs):
        super(FIA, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies FIA'
        self.report_name = 'CNMC INVENTARI FIA'

    def get_sequence(self):
        search_params = [('inventari', '=', 'fiabilitat')]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCellesCella.search(
            search_params, 0, 0, False, {'active_test': False})

    def consumer(self):
        O = self.connection
        fields_to_read = ['name', 'cini', 'tipus_element',
                          'cnmc_tipo_instalacion', 'installacio', 'data_pm']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                cll = O.GiscedataCellesCella.read(item, fields_to_read)

                #Comprovar si es tipus fiabilitat
                if cll['tipus_element']:

                    cllt = O.GiscedataCellesTipusElement.read(
                        cll['tipus_element'][0], ['name'])

                codi = cll['cnmc_tipo_instalacion']

                #Instal·lació a la que pertany
                cllinst = cll['installacio'].split(',')

                data_pm = ''
                if cll['data_pm']:
                    data_pm_ct = datetime.strptime(str(cll['data_pm']),
                                                   '%Y-%m-%d')
                    data_pm = data_pm_ct.strftime('%d/%m/%Y')

                #Per trobar la comunitat autonoma
                ccaa = ''
                element_act = ''
                #Comprovo si la cella pertany a ct o lat per trobar la ccaa
                if cllinst[0] == 'giscedata.cts':
                    ct_vals = O.GiscedataCts.read(int(cllinst[1]),
                                                  ['id_municipi', 'name'])
                    if ct_vals['id_municipi']:
                        id_municipi = ct_vals['id_municipi'][0]
                    element_act = ct_vals['name']

                elif cllinst[0] == 'giscedata.at.suport':
                    linia_vals = O.GiscedataAtSuport.read(int(cllinst[1]),
                                                          ['linia'])
                    linia_id = int(linia_vals['linia'][0])
                    linia_name = linia_vals['linia'][1]
                    lat_vals = O.GiscedataAtLinia.read(linia_id, ['municipi'])
                    if lat_vals['municipi']:
                        id_municipi = lat_vals['municipi'][0]
                    element_act = linia_name

                if id_municipi:
                    ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi(
                        id_municipi)[0]

                output = [
                    '%s' % cll['name'],
                    cll['cini'] or '',
                    element_act,
                    codi or '',
                    ccaa or '',
                    data_pm,
                ]
                self.output_q.put(output)
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


from F1 import F1
from F1bis import F1bis
from F9 import F9
from F10AT import F10AT
from F10BT import F10BT
from F11 import F11
from F12 import F12
from F12bis import F12bis
from F13 import F13
from F13bis import F13bis
from F13C import F13c
from F14 import F14
from F15 import F15
from F16 import F16
from F20 import F20
from create_celles import CreateCelles4_2015



from cnmcmodel import CNMCModel


class F7Res4771(CNMCModel):
    """
        Class for seventh file of resolution 4771(Fiabilidad)
    """

    fields = [
        'identificador',
        'cini',
        'elemento_act',
        'codigo_tipo_inst',
        'codigo_ccaa',
        'fecha_aps'
    ]

    @property
    def ref(self):
        return self.store['identificador']

    def __cmp__(self, other):
        comp_fields = ['cini', 'elemento_act']
        if self.diff(other, comp_fields):
            return True
        else:
            return False








# -*- coding: utf-8 -*-
from datetime import datetime
import traceback

from libcnmc.utils import get_ine, format_f, convert_srid, get_srid
from libcnmc.core import MultiprocessBased


class F11(MultiprocessBased):
    def __init__(self, **kwargs):
        super(F11, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F11 - CTS'
        self.base_object = 'CTS'

    def get_sequence(self):
        search_params = [('id_installacio.name', '!=', 'SE')]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += ['|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCts.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_node_vertex(self, ct_id):
        O = self.connection
        bloc = O.GiscegisBlocsCtat.search([('ct', '=', ct_id)])
        node = ''
        vertex = None
        if bloc:
            bloc = O.GiscegisBlocsCtat.read(bloc[0], ['node', 'vertex'])
            node = bloc['node'][1]
            if bloc['vertex']:
                v = O.GiscegisVertex.read(bloc['vertex'][0], ['x', 'y'])
                vertex = (round(v['x'], 3), round(v['y'], 3))
        return node, vertex

    def get_ine(self, municipi_id):
        O = self.connection
        muni = O.ResMunicipi.read(municipi_id, ['ine', 'dc'])
        return get_ine(O, muni['ine'])

    def get_sortides_ct(self, ct_name):
        O = self.connection
        search = '%s-' % ct_name
        sortides = O.GiscegisBlocsFusiblesbt.search(
            [('codi', 'ilike', search)]
        )
        disponibles = len(sortides)
        utilitzades = 0
        for sortida in O.GiscegisBlocsFusiblesbt.read(sortides, ['node']):
            if sortida['node']:
                node = sortida['node'][0]
                edges = O.GiscegisEdge.search(
                    ['|', ('start_node', '=', node), ('end_node', '=', node)]
                )
                if len(edges) > 1:
                    utilitzades += 1
        return utilitzades, disponibles

    def get_tipus(self, subtipus_id):
        o = self.connection
        tipus = ''
        subtipus = o.GiscedataCtsSubtipus.read(subtipus_id, ['categoria_cne'])
        if subtipus['categoria_cne']:
            cne_id = subtipus['categoria_cne'][0]
            cne = o.GiscedataCneCtTipus.read(cne_id, ['codi'])
            tipus = cne['codi']
        return tipus

    def get_saturacio(self, ct_id):
        o = self.connection
        saturacio = ''
        if 'giscedata.transformadors.saturacio' in o.models:
            sat_obj = o.GiscedataTransformadorsSaturacio
            trafo_obj = o.GiscedataTransformadorTrafo
            sat = sat_obj.search([
                ('ct.id', '=', ct_id)
            ])
            saturacio = 0
            for sat in sat_obj.read(sat, ['b1_b2', 'trafo']):
                trafo = trafo_obj.read(sat['trafo'][0], ['potencia_nominal'])
                saturacio += trafo['potencia_nominal'] * sat['b1_b2']
            saturacio *= 0.9
        return saturacio

    def get_potencia_trafos(self, id_ct):
        o = self.connection
        res = 0
        ids_trafos = o.GiscedataTransformadorTrafo.search([
            ('ct', '=', id_ct), ('id_estat.cnmc_inventari', '=', True)])
        if ids_trafos:
            for elem in ids_trafos:
                trafo = o.GiscedataTransformadorTrafo.read(
                    elem, ['potencia_nominal'])
                if trafo:
                    res += trafo['potencia_nominal']
        return res

    def consumer(self):
        o_codi_r1 = 'R1-%s' % self.codi_r1[-3:]
        O = self.connection
        fields_to_read = [
            'name', 'cini', 'id_municipi',   'tensio_p', 'id_subtipus',
            'perc_financament', 'propietari', 'numero_maxim_maquines',
            'potencia'
        ]
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                ct = O.GiscedataCts.read(item, fields_to_read)
                o_node, vertex = self.get_node_vertex(item)
                o_ct = ct['name']
                o_cini = ct['cini'] or ''
                if ct['id_municipi']:
                    o_ine_prov, o_ine_muni = self.get_ine(ct['id_municipi'][0])
                else:
                    o_ine_muni, o_ine_prov = '', ''

                try:
                    o_tensio_p = format_f(
                        float(ct['tensio_p']) / 1000.0, decimals=3) or ''
                except:
                    o_tensio_p = ''

                if ct['id_subtipus']:
                    o_tipo = self.get_tipus(ct['id_subtipus'][0])
                else:
                    o_tipo = ''
                o_potencia = format_f(self.get_potencia_trafos(item),
                                      decimals=3)
                cups = O.GiscedataCupsPs.search([('et', '=', ct['name'])])
                o_energia = sum(
                    x['cne_anual_activa']
                    for x in O.GiscedataCupsPs.read(
                        cups, ['cne_anual_activa']
                    )
                )
                o_pic_activa = format_f(
                    self.get_saturacio(ct['id']), decimals=3)
                o_pic_reactiva = ''
                o_s_utilitades, o_s_disponibles = self.get_sortides_ct(
                    ct['name']
                )
                o_propietari = int(ct['propietari'])
                o_num_max_maquines = ct['numero_maxim_maquines']
                o_incorporacio = self.year
                x = ''
                y = ''
                z = ''
                res_srid = ['', '']
                if vertex:
                    res_srid = convert_srid(
                        self.codi_r1, get_srid(O), vertex)
                self.output_q.put([
                    o_node,
                    o_ct,
                    o_cini,
                    format_f(res_srid[0], decimals=3),
                    format_f(res_srid[1], decimals=3),
                    z,
                    o_ine_muni,
                    o_ine_prov,
                    o_tensio_p,
                    o_tipo,
                    o_potencia,
                    format_f(o_energia, decimals=3),
                    o_pic_activa,
                    o_pic_reactiva,
                    o_s_utilitades,
                    o_s_disponibles,
                    o_codi_r1,
                    o_propietari,
                    o_num_max_maquines,
                    o_incorporacio
                ])
            except:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()
